#!/usr/bin/env python
# -*- coding: utf-8 -*-
try:
    import psutil
except ImportError:
    print('Error: psutil module not found!')
    exit()
import time
import argparse
import sys
from datetime import datetime as dt
import numpy as np

msg_info = []
monitor_ports=[]

def main():
    args = parsing_parameters()
    read_msginfo(args)    
    get_monitor_ports()
   
    port_send_aves = []
    port_recv_aves = []
    average_count = 0
    interval= int (args.interval) - 1  
   
    while True:
      try:
        #sampleing    
        net_recv, net_send, currTime = get_rate(get_port_data)
        if average_count == 0:
          print_title()
        
        index = 0
        for info in msg_info:
          portpair = info[0][0]
          
          port = monitor_ports[index]
          if average_count == 0:
            if net_send:
              port_send_aves.append( net_send.get(port) )
              port_recv_aves.append( net_recv.get(port) )
            else:
              port_send_aves.append( 0 )
              port_recv_aves.append( 0 )
            
          else:
            if net_send:
              port_send_aves[index] = ( average_count*port_send_aves[index] + net_send.get(port) )/( average_count + 1) 
              port_recv_aves[index] = ( average_count*port_recv_aves[index] + net_recv.get(port) )/( average_count + 1) 
          
          if interval == average_count:
              ave_send = float('%.2f' %(port_send_aves[index]) )  
              ave_recv = float('%.2f' %(port_recv_aves[index]) )  
              print('%-5s%-40s%-8s%-8s' % (index, portpair, ave_send, ave_recv ) )
                    
          index = index + 1
        
        if interval == average_count :
          t = np.sum(port_send_aves)
          send_total = float('%.2f' %(t) )
          
          nave = np.array(port_send_aves)
          non_zeros = (nave != 0 ).sum()
          if non_zeros == 0:
            a = 0
          else:
            a = t/non_zeros
            
          average_send_ave =  float('%.2f' %(a) )
          
          t = np.sum(port_recv_aves)
          recv_total = float('%.2f' %(t) )
          
          nave = np.array(port_recv_aves)
          non_zeros = (nave != 0 ).sum()
          if non_zeros == 0:
            a = 0
          else:
            a = t/non_zeros
            
          average_recv_ave =  float('%.2f' %(a) )          
          print_footprint(average_send_ave, average_recv_ave, send_total, recv_total)
          
          average_count = 0
          port_send_aves = []
          port_recv_aves = []
          
          break
        else:
          average_count = average_count + 1
        
                                    
      except KeyboardInterrupt:
        exit()      

def get_monitor_ports():
    for info in msg_info:
      port1,port2 = info[0][0].split(':')
      monitor_ports.append( "zve_"+port1 )
    #print (monitor_ports)
    
def read_msginfo(args):
    idxs = args.ppindex.split(",")
    indices = []
    for i in idxs:
      indices.append(int(i))  
    #print(indices)
    
    aidx = 0
    fidx = 0
    with open( args.cfg, 'r' ) as fp:
        for portpair in fp:
          fidx = fidx + 1          
          portpair = portpair.strip('\n')
          
          addit = 0
          if indices[0] == 0:
            addit = 1
          
          if fidx in indices:
            addit = 1
          
          if addit == 1:
            msg_info.append([])
            msg_info[aidx].append([portpair])
            aidx = aidx + 1
            
        #print(msg_info)

def parsing_parameters():
    argparse.ArgumentParser(add_help=False)
    main_parser = argparse.ArgumentParser()
    main_parser.add_argument("-p", "--ppindex", required=False, default="0", help="portpair indexes, 1,3,4. 0 for all")
    main_parser.add_argument("-c", "--cfg", required=False, default="/etc/port_pairs", help="msg portpair configuration file")
    main_parser.add_argument("-i", "--interval", required=False, default="5", help="counts between periodic bandwidth reports")
    return main_parser.parse_args()

def get_port_data():
    net_io_info = {}
    port_info = {}
    recv = {}
    sent = {}
    currTime = {}

    net_io_info = psutil.net_io_counters(pernic=True)
    port_info = net_io_info.keys()
    
    for port in port_info:
        if port in monitor_ports:
          currTime.setdefault(port, time.time())
          recv.setdefault(port, net_io_info.get(port).bytes_recv)
          sent.setdefault(port, net_io_info.get(port).bytes_sent)

    return recv, sent, currTime

def get_rate(func):

    old_recv, old_sent, ts_start = func()
    time.sleep(1)
    now_recv, now_sent, ts_end = func()

    net_recv = {}
    net_send = {}
    timeDiff = {}

    for port in monitor_ports:
        if now_recv:
          timeDiff.setdefault(port, ts_end.get(port) - ts_start.get(port))
          
          rate =  float('%.2f' %((now_recv.get(port) - old_recv.get(port))*8 / (1024*1024*timeDiff.get(port))))
          if (rate < 1 ):
            rate = 0
          net_recv.setdefault(port,rate)  
          
          rate =  float('%.2f' %((now_sent.get(port) - old_sent.get(port))*8 / (1024*1024*timeDiff.get(port))))
          if (rate < 1 ):
            rate = 0            
          net_send.setdefault(port, rate) #Byte to Mbps

    return net_recv, net_send, timeDiff
    
def print_title():
    print('================================================================')
    print('\t%+32s' %('Performance Data'))
    print('================================================================')
    print('%-5s%-38s%-25s' %('No.','PortPair','Throughput(in Mbps)'))
    print('\t%+52s' %('Send    Receive'))
    print('----------------------------------------------------------------')

def print_footprint(send_average, recv_average, send_total, recv_total):
    print('----------------------------------------------------------------')
    #print('\t%+32s%+9s%+9s' %('Total Throughput(Mbps):', send_total, recv_total) )
    #print('\t%+32s%+9s%+9s' %('Average Throughput(Mbps):', send_average, recv_average) )

    print('\t%+55s\n'% str(dt.now()))

if __name__ == "__main__":
    main()

